# -*- coding: utf-8 -*-
from pip_services4_data.query import FilterParams, PagingParams
from eic_aichat_users.accounts.data.AccountV1 import AccountV1
from eic_aichat_users.accounts.persistence.IAccountsPersistence import IAccountsPersistence


ACCOUNT1 = AccountV1('1', 'user1@conceptual.vision', 'Test User 1')
ACCOUNT2 = AccountV1('2', 'user2@conceptual.vision', 'Test User 2')
ACCOUNT3 = AccountV1('3', 'user3@conceptual.vision', 'Test User 3')


class AccountsPersistenceFixture:
    _persistence: IAccountsPersistence

    def __init__(self, persistence: IAccountsPersistence):
        assert persistence is not None
        self._persistence = persistence

    def __create_accounts(self):
        # Create the first account
        account = self._persistence.create(None, ACCOUNT1)
        assert ACCOUNT1.id == account.id
        assert ACCOUNT1.login == account.login
        assert ACCOUNT1.name == account.name
        assert account.active is True

        # Create the second account
        account = self._persistence.create(None, ACCOUNT2)
        assert ACCOUNT2.id == account.id
        assert ACCOUNT2.login == account.login
        assert ACCOUNT2.name == account.name
        assert account.active is True

        # Create the third account
        account = self._persistence.create(None, ACCOUNT3)
        assert ACCOUNT3.id == account.id
        assert ACCOUNT3.login == account.login
        assert ACCOUNT3.name == account.name
        assert account.active is True

    def test_crud_operations(self):
        # Create accounts
        self.__create_accounts()

        # Get all accounts
        page = self._persistence.get_page_by_filter(None, FilterParams(), PagingParams())
        assert page is not None
        assert len(page.data) == 3

        account1 = page.data[0]

        # Update the account
        account1.name = 'Updated User 1'
        account = self._persistence.update(None, account1)

        assert account.id == account1.id
        assert account.name == 'Updated User 1'
        assert account.login == account1.login

        # Delete the account
        self._persistence.delete_by_id(None, ACCOUNT1.id)

        # Try to get deleted account
        deleted = self._persistence.get_one_by_id(None, ACCOUNT1.id)
        assert deleted is None

    def test_get_with_filter(self):
        # Create accounts
        self.__create_accounts()

        # Get accounts by search
        page = self._persistence.get_page_by_filter(
            None,
            FilterParams.from_tuples('active', True, 'search', 'user'),
            PagingParams()
        )
        assert page is not None
        assert len(page.data) == 3

        # Get account by login
        account = self._persistence.get_one_by_login(None, ACCOUNT2.login)
        assert account is not None
        assert account.id == ACCOUNT2.id

        # Get account by id or login
        account = self._persistence.get_one_by_id_or_login(None, ACCOUNT3.login)
        assert account is not None
        assert account.id == ACCOUNT3.id



# -*- coding: utf-8 -*-
from typing import List, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable


class UserRolesV1(IStringIdentifiable):
    def __init__(
        self,
        id: Optional[str] = None,
        roles: Optional[List[str]] = None
    ):
        self.id: str = id
        self.roles: List[str] = roles or []
        self.update_time: datetime = datetime.now()

    def to_dict(self):
        return self.__dict__



# -*- coding: utf-8 -*-
from pip_services4_components.config import ConfigParams
from pip_services4_data.query import FilterParams, PagingParams

from eic_aichat_users.settings.data.SettingsSectionV1 import SettingsSectionV1
from eic_aichat_users.settings.persistence.ISettingsPersistence import ISettingsPersistence


class SettingsPersistenceFixture:
    def __init__(self, persistence: ISettingsPersistence):
        assert persistence is not None
        self._persistence = persistence

    def test_get_and_set(self):
        settings = self._persistence.set(
            None,
            SettingsSectionV1(
                id='test.1',
                parameters=ConfigParams.from_tuples(
                    'key1', 'value11',
                    'key2', 'value12'
                )
            )
        )

        assert settings is not None
        assert settings.id == 'test.1'
        assert settings.parameters.get_as_string('key1') == 'value11'

        settings = self._persistence.set(
            None,
            SettingsSectionV1(
                id='test.2',
                parameters=ConfigParams.from_tuples(
                    'key1', 'value21',
                    'key2', 'value22'
                )
            )
        )

        assert settings is not None
        assert settings.id == 'test.2'
        assert settings.parameters.get_as_string('key1') == 'value21'

        settings = self._persistence.get_one_by_id(None, 'test.1')

        assert settings is not None
        assert settings.id == 'test.1'
        assert settings.parameters.get_as_string('key1') == 'value11'

        page = self._persistence.get_page_by_filter(None, FilterParams.from_tuples('id_starts', 'test'), None)
        assert len(page.data) == 2

    def test_set_parameter(self):
        settings = self._persistence.modify(None, 'test.1', ConfigParams.from_tuples('key1', 'value11a'), None)
        assert settings.parameters.get_as_string('key1') == 'value11a'

        settings = self._persistence.modify(None, 'test.1', ConfigParams.from_tuples('key1', 'value11b'), None)
        assert settings.parameters.get_as_string('key1') == 'value11b'

        settings = self._persistence.get_one_by_id(None, 'test.1')
        assert settings.parameters.get_as_string('key1') == 'value11b'

        settings = self._persistence.modify(None, 'test.1', ConfigParams.from_tuples('key1.key11', 'value11a'), None)
        assert settings.parameters.get_as_string('key1.key11') == 'value11a'

    def test_increment_parameter(self):
        settings = self._persistence.modify(None, 'test.1', None, ConfigParams.from_tuples('key1', 1))
        assert settings.parameters.get_as_string('key1') == '1'

        settings = self._persistence.modify(None, 'test.1', None, ConfigParams.from_tuples('key1', 2))
        assert settings.parameters.get_as_string('key1') == '3'

        settings = self._persistence.get_one_by_id(None, 'test.1')
        assert settings.parameters.get_as_string('key1') == '3'

    def test_get_sections(self):
        settings = self._persistence.set(
            None,
            SettingsSectionV1(
                id='test.1',
                parameters=ConfigParams.from_tuples(
                    'key1', 'value11',
                    'key2', 'value12'
                )
            )
        )

        assert settings.id == 'test.1'
        assert settings.parameters.get_as_string('key1') == 'value11'

        page = self._persistence.get_page_by_filter(None, FilterParams(), PagingParams(0, 10, True))
        assert len(page.data) == 1



# -*- coding: utf-8 -*-
from typing import Optional, List

from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_rpc.commands import CommandSet, ICommandable

from ..data import PartyActivityV1
from ..persistence import IActivitiesPersistence

from .ActivitiesCommandSet import ActivitiesCommandSet
from .IActivitiesService import IActivitiesService

class ActivitiesService(IActivitiesService, IReferenceable, ICommandable):
    _persistence: IActivitiesPersistence = None
    __command_set: ActivitiesCommandSet = None

    def set_references(self, references: IReferences):
        self._persistence = references.get_one_required(
            Descriptor('aichatusers-activities', 'persistence', '*', '*', '1.0')
        )

    def get_command_set(self) -> CommandSet:
        if self.__command_set is None:
            self.__command_set = ActivitiesCommandSet(self)

        return self.__command_set

    def get_activities(self, context: IContext, filter_params: FilterParams, paging_params: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging_params)

    def log_activity(self, context: IContext, activity: PartyActivityV1) -> PartyActivityV1:
        return self._persistence.create(context, activity)

    def batch_log_activities(self, context: IContext, activities: List[PartyActivityV1]) -> None:
        if activities is None or len(activities) == 0:
            return

        for activity in activities:
            self._persistence.create(context, activity)

    def delete_activities_by_filter(self, context: IContext, filter_params: FilterParams) -> None:
        self._persistence.delete_by_filter(context, filter_params)


# -*- coding: utf-8 -*-
from typing import List, Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data.UserRolesV1 import UserRolesV1
from ..persistence.IRolesPersistence import IRolesPersistence
from .IRolesService import IRolesService


class RolesService(IRolesService, IConfigurable, IReferenceable):
    _persistence: IRolesPersistence = None

    def configure(self, config: ConfigParams):
        pass  # No config needed yet

    def set_references(self, references: IReferences):
        self._persistence = references.get_one_required(
            Descriptor('aichatusers-roles', 'persistence', '*', '*', '1.0')
        )

    def get_roles_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                            paging: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging)

    def get_roles_by_id(self, context: Optional[IContext], user_id: str) -> Optional[List[str]]:
        roles = self._persistence.get_one_by_id(context, user_id)
        return roles.roles if roles else None

    def set_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        item = UserRolesV1(id=user_id, roles=roles)
        result = self._persistence.set(context, item)
        return result.roles if result else []

    def grant_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        if not roles:
            return []

        existing_roles = self.get_roles_by_id(context, user_id) or []

        # Add new roles avoiding duplicates
        updated_roles = list(set(existing_roles + roles))

        return self.set_roles(context, user_id, updated_roles)

    def revoke_roles(self, context: Optional[IContext], user_id: str, roles: List[str]) -> List[str]:
        if not roles:
            return []

        existing_roles = self.get_roles_by_id(context, user_id) or []

        # Remove specified roles
        updated_roles = [r for r in existing_roles if r not in roles]

        return self.set_roles(context, user_id, updated_roles)

    def authorize(self, context: Optional[IContext], user_id: str, roles: List[str]) -> bool:
        if not roles:
            return False

        existing_roles = self.get_roles_by_id(context, user_id) or []

        return all(role in existing_roles for role in roles)



# -*- coding: utf-8 -*-
from pip_services4_components.refer import Descriptor
from pip_services4_components.build import Factory

from eic_aichat_users.accounts.logic.AccountsService import AccountsService
from eic_aichat_users.accounts.persistence.AccountsMemoryPersistence import AccountsMemoryPersistence
from eic_aichat_users.accounts.persistence.AccountsMongoDbPersistence import AccountsMongoDbPersistence

from eic_aichat_users.settings.logic.SettingsService import SettingsService
from eic_aichat_users.settings.persistence.SettingsMemoryPersistence import SettingsMemoryPersistence
from eic_aichat_users.settings.persistence.SettingsMongoDbPersistence import SettingsMongoDbPersistence

from eic_aichat_users.passwords.logic.PasswordsService import PasswordsService
from eic_aichat_users.passwords.persistence.PasswordsMemoryPersistence import PasswordsMemoryPersistence
from eic_aichat_users.passwords.persistence.PasswordsMongoDbPersistence import PasswordsMongoDbPersistence

from eic_aichat_users.activities.logic.ActivitiesService import ActivitiesService
from eic_aichat_users.activities.persistence.ActivitiesMemoryPersistence import ActivitiesMemoryPersistence
from eic_aichat_users.activities.persistence.ActivitiesMongoDbPersistence import ActivitiesMongoDbPersistence

from eic_aichat_users.roles.logic.RolesService import RolesService
from eic_aichat_users.roles.persistence.RolesMemoryPersistence import RolesMemoryPersistence
from eic_aichat_users.roles.persistence.RolesMongoDbPersistence import RolesMongoDbPersistence

from eic_aichat_users.sessions.logic.SessionsService import SessionsService
from eic_aichat_users.sessions.persistence.SessionsMemoryPersistence import SessionsMemoryPersistence
from eic_aichat_users.sessions.persistence.SessionsMongoDbPersistence import SessionsMongoDbPersistence


class AIChatUsersFactory(Factory):
    __AccountsMemoryPersistenceDescriptor = Descriptor('aichatusers-accounts', 'persistence', 'memory', '*', '1.0')
    __AccountsMongoDbPersistenceDescriptor = Descriptor('aichatusers-accounts', 'persistence', 'mongodb', '*', '1.0')
    __AccountsServiceDescriptor = Descriptor('aichatusers-accounts', 'service', 'default', '*', '1.0')

    __SettingsMemoryPersistenceDescriptor = Descriptor('aichatusers-settings', 'persistence', 'memory', '*', '1.0')
    __SettingsMongoDbPersistenceDescriptor = Descriptor('aichatusers-settings', 'persistence', 'mongodb', '*', '1.0')
    __SettingsServiceDescriptor = Descriptor('aichatusers-settings', 'service', 'default', '*', '1.0')

    __PasswordsMemoryPersistenceDescriptor = Descriptor('aichatusers-passwords', 'persistence', 'memory', '*', '1.0')
    __PasswordsMongoDbPersistenceDescriptor = Descriptor('aichatusers-passwords', 'persistence', 'mongodb', '*', '1.0')
    __PasswordsServiceDescriptor = Descriptor('aichatusers-passwords', 'service', 'default', '*', '1.0')

    __ActivitiesMemoryPersistenceDescriptor = Descriptor('aichatusers-activities', 'persistence', 'memory', '*', '1.0')
    __ActivitiesMongoDbPersistenceDescriptor = Descriptor('aichatusers-activities', 'persistence', 'mongodb', '*', '1.0')
    __ActivitiesServiceDescriptor = Descriptor('aichatusers-activities', 'service', 'default', '*', '1.0')

    __RolesMemoryPersistenceDescriptor = Descriptor('aichatusers-roles', 'persistence', 'memory', '*', '1.0')
    __RolesMongoDbPersistenceDescriptor = Descriptor('aichatusers-roles', 'persistence', 'mongodb', '*', '1.0')
    __RolesServiceDescriptor = Descriptor('aichatusers-roles', 'service', 'default', '*', '1.0')

    __SessionsMemoryPersistenceDescriptor = Descriptor('aichatusers-sessions', 'persistence', 'memory', '*', '1.0')
    __SessionsMongoDbPersistenceDescriptor = Descriptor('aichatusers-sessions', 'persistence', 'mongodb', '*', '1.0')
    __SessionsServiceDescriptor = Descriptor('aichatusers-sessions', 'service', 'default', '*', '1.0')


    def __init__(self):
        super().__init__()

        self.register_as_type(self.__AccountsMemoryPersistenceDescriptor, AccountsMemoryPersistence)
        self.register_as_type(self.__AccountsMongoDbPersistenceDescriptor, AccountsMongoDbPersistence)
        self.register_as_type(self.__AccountsServiceDescriptor, AccountsService)
        
        self.register_as_type(self.__SettingsMemoryPersistenceDescriptor, SettingsMemoryPersistence)
        self.register_as_type(self.__SettingsMongoDbPersistenceDescriptor, SettingsMongoDbPersistence)
        self.register_as_type(self.__SettingsServiceDescriptor, SettingsService)

        self.register_as_type(self.__PasswordsMemoryPersistenceDescriptor, PasswordsMemoryPersistence)
        self.register_as_type(self.__PasswordsMongoDbPersistenceDescriptor, PasswordsMongoDbPersistence)
        self.register_as_type(self.__PasswordsServiceDescriptor, PasswordsService)

        self.register_as_type(self.__ActivitiesMemoryPersistenceDescriptor, ActivitiesMemoryPersistence)
        self.register_as_type(self.__ActivitiesMongoDbPersistenceDescriptor, ActivitiesMongoDbPersistence)
        self.register_as_type(self.__ActivitiesServiceDescriptor, ActivitiesService)

        self.register_as_type(self.__RolesMemoryPersistenceDescriptor, RolesMemoryPersistence)
        self.register_as_type(self.__RolesMongoDbPersistenceDescriptor, RolesMongoDbPersistence)
        self.register_as_type(self.__RolesServiceDescriptor, RolesService)

        self.register_as_type(self.__SessionsMemoryPersistenceDescriptor, SessionsMemoryPersistence)
        self.register_as_type(self.__SessionsMongoDbPersistenceDescriptor, SessionsMongoDbPersistence)
        self.register_as_type(self.__SessionsServiceDescriptor, SessionsService)



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Callable, Optional
from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_components.config import ConfigParams
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from ..data.SettingsSectionV1 import SettingsSectionV1
from .ISettingsPersistence import ISettingsPersistence


class SettingsMemoryPersistence(IdentifiableMemoryPersistence, ISettingsPersistence):

    def __init__(self):
        super().__init__()

    def __compose_filter(self, filter_params: FilterParams) -> Callable[[SettingsSectionV1], bool]:
        filter_params = filter_params or FilterParams()
        search = filter_params.get_as_nullable_string('search')
        id = filter_params.get_as_nullable_string('id')
        id_starts = filter_params.get_as_nullable_string('id_starts')

        def filter_fn(item: SettingsSectionV1) -> bool:
            if search and search.lower() not in (item.id or '').lower():
                return False
            if id and item.id != id:
                return False
            if id_starts and not (item.id or '').startswith(id_starts):
                return False
            return True

        return filter_fn

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter_params), paging, None, None)

    def set(self, context: Optional[IContext], item: SettingsSectionV1) -> SettingsSectionV1:
        item.update_time = item.update_time or datetime.now()
        return super().set(context, item)

    def modify(self, context: Optional[IContext], id: str,
               update_params: ConfigParams, increment_params: ConfigParams) -> SettingsSectionV1:
        index = next((i for i, x in enumerate(self._items) if x.id == id), -1)
        item = self._items[index] if index >= 0 else SettingsSectionV1(id)

        if update_params:
            for key, value in update_params.items():
                item.parameters.set_as_object(key, value)

        if increment_params:
            for key, inc in increment_params.items():
                current = item.parameters.get_as_long_with_default(key, 0)
                item.parameters.set_as_object(key, current + int(inc))

        item.update_time = datetime.now()

        if index < 0:
            self._items.append(item)

        self._logger.trace(context, "Modified item with id=%s", id)
        self.save(context)
        return item



# -*- coding: utf-8 -*-
from pip_services4_components.config import ConfigParams
from pip_services4_components.refer import References, Descriptor
from pip_services4_data.query import FilterParams

from eic_aichat_users.roles.logic.RolesService import RolesService
from eic_aichat_users.roles.persistence.RolesMemoryPersistence import RolesMemoryPersistence

ROLES = ['Role 1', 'Role 2', 'Role 3']


class TestRolesService:
    persistence: RolesMemoryPersistence
    service: RolesService

    def setup_method(self):
        self.persistence = RolesMemoryPersistence()
        self.persistence.configure(ConfigParams())

        self.service = RolesService()
        self.service.configure(ConfigParams())

        references = References.from_tuples(
            Descriptor('aichatusers-roles', 'persistence', 'memory', 'default', '1.0'), self.persistence,
            Descriptor('aichatusers-roles', 'service', 'default', 'default', '1.0'), self.service
        )

        self.service.set_references(references)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_get_and_set_roles(self):
        roles = self.service.set_roles(None, '1', ROLES)
        assert roles is not None
        assert len(roles) == 3

        roles = self.service.get_roles_by_id(None, '1')
        assert roles is not None
        assert len(roles) == 3

    def test_grant_and_revoke_roles(self):
        roles = self.service.grant_roles(None, '1', ['Role 1'])
        assert roles == ['Role 1']

        roles = self.service.grant_roles(None, '1', ['Role 1', 'Role 2', 'Role 3'])
        assert sorted(roles) == sorted(ROLES)

        roles = self.service.revoke_roles(None, '1', ['Role 1'])
        assert sorted(roles) == ['Role 2', 'Role 3']

        roles = self.service.revoke_roles(None, '1', ['Role 1', 'Role 2'])
        assert roles == ['Role 3']

    def test_authorize_roles(self):
        self.service.grant_roles(None, '1', ['Role 1', 'Role 2'])

        result = self.service.authorize(None, '1', ['Role 1'])
        assert result is True

        result = self.service.authorize(None, '1', ['Role 2', 'Role 3'])
        assert result is False



# -*- coding: utf-8 -*-
from pip_services4_components.config import ConfigParams
from pip_services4_components.refer import References, Descriptor
from pip_services4_data.query import PagingParams, FilterParams

from eic_aichat_users.accounts.data.AccountV1 import AccountV1
from eic_aichat_users.accounts.logic.AccountsService import AccountsService
from eic_aichat_users.accounts.persistence.AccountsMemoryPersistence import AccountsMemoryPersistence


ACCOUNT1 = AccountV1('1', 'user1@conceptual.vision', 'Test User 1')
ACCOUNT2 = AccountV1('2', 'user2@conceptual.vision', 'Test User 2')


class TestAccountsService:
    persistence: AccountsMemoryPersistence
    service: AccountsService

    def setup_method(self):
        self.persistence = AccountsMemoryPersistence()
        self.persistence.configure(ConfigParams())

        self.service = AccountsService()
        self.service.configure(ConfigParams.from_tuples(
            'options.login_as_email', True
        ))

        references = References.from_tuples(
            Descriptor('aichatusers-accounts', 'persistence', 'memory', 'default', '1.0'), self.persistence,
            Descriptor('aichatusers-accounts', 'service', 'default', 'default', '1.0'), self.service
        )

        self.service.set_references(references)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_create_new_account(self):
        account = self.service.create_account(None, ACCOUNT1)
        assert account is not None
        assert account.name == ACCOUNT1.name
        assert account.login == ACCOUNT1.login

    def test_fail_to_create_duplicate_account(self):
        # First creation
        account = self.service.create_account(None, ACCOUNT1)
        assert account is not None

        # Second creation with same login should fail
        error = None
        try:
            self.service.create_account(None, ACCOUNT1)
        except Exception as e:
            error = e

        assert error is not None

    def test_get_accounts(self):
        account1 = self.service.create_account(None, ACCOUNT1)
        account2 = self.service.create_account(None, ACCOUNT2)

        # Get account by ID
        result = self.service.get_account_by_id(None, account1.id)
        assert result is not None
        assert result.id == account1.id

        # Get account by login
        result = self.service.get_account_by_id_or_login(None, account2.login)
        assert result is not None
        assert result.id == account2.id

        # Get all accounts
        page = self.service.get_accounts(None, FilterParams(), PagingParams())
        assert page is not None
        assert len(page.data) == 2

    def test_update_account(self):
        account = self.service.create_account(None, ACCOUNT1)
        account.name = 'Updated Name'
        updated = self.service.update_account(None, account)

        assert updated is not None
        assert updated.name == 'Updated Name'

    def test_change_account_email(self):
        account = self.service.create_account(None, ACCOUNT1)
        account.login = 'new@email.com'
        account.name = 'New Name'

        updated = self.service.update_account(None, account)
        assert updated is not None
        assert updated.login == 'new@email.com'
        assert updated.name == 'New Name'



# -*- coding: utf-8 -*-
import datetime

from pip_services4_components.context import Context
from pip_services4_data.query import FilterParams, PagingParams
from pip_services4_components.refer import References, Descriptor

from eic_aichat_users.activities.data import PartyActivityV1, ReferenceV1
from eic_aichat_users.activities.logic import ActivitiesService
from eic_aichat_users.activities.persistence import ActivitiesMemoryPersistence


class TestActivitiesService:
    activity1 = PartyActivityV1(
        id='1',
        time=datetime.datetime.now(),
        type='test',
        party=ReferenceV1(id='1', type='party'),
        ref_item=ReferenceV1(id='1', type='item'),
        ref_parents=[],
        ref_party=None,
        details={}
    )
    activity2 = PartyActivityV1(
        id='2',
        time=datetime.datetime.now(),
        type='test',
        party=ReferenceV1(id='1', type='party'),
        ref_item=ReferenceV1(id='2', type='item'),
        ref_parents=[],
        ref_party=None,
        details={}
    )
    activity3 = PartyActivityV1(
        id='3',
        time=datetime.datetime.now(),
        type='test',
        party=ReferenceV1(id='2', type='party'),
        ref_item=ReferenceV1(id='2', type='item'),
        ref_parents=[],
        ref_party=None,
        details={}
    )

    def setup_method(self):
        self.persistence = ActivitiesMemoryPersistence()
        self.service = ActivitiesService()
        
        self.references = References.from_tuples(
            Descriptor('activities', 'persistence', 'memory', 'default', '1.0'), self.persistence,
            Descriptor('activities', 'service', 'default', 'default', '1.0'), self.service
        )

        self.service.set_references(self.references)
        self.service

    def test_get_activities(self):
        # Create activities
        self.service.log_activity(Context.from_trace_id('123'), self.activity1)
        self.service.log_activity(Context.from_trace_id('123'), self.activity2)
        self.service.log_activity(Context.from_trace_id('123'), self.activity3)

        # Get all activities
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams(),
            PagingParams()
        )
        assert len(page.data) == 3

        # Get activities by party id
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('party_id', '1'),
            PagingParams()
        )
        assert len(page.data) == 2

        # Get activities by ref item id
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('ref_item_id', '2'),
            PagingParams()
        )
        assert len(page.data) == 2

    def test_log_activity(self):
        # Log activity
        activity = self.service.log_activity(Context.from_trace_id('123'), self.activity1)
        assert activity is not None
        assert activity.id == self.activity1.id
        assert activity.type == self.activity1.type
        assert activity.party.id == self.activity1.party.id

        # Get activity by id
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('id', activity.id),
            PagingParams()
        )
        assert len(page.data) == 1

    def test_batch_log_activities(self):
        # Batch log activities
        self.service.batch_log_activities(
            Context.from_trace_id('123'),
            [self.activity1, self.activity2]
        )

        # Get all activities
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams(),
            PagingParams()
        )
        assert len(page.data) == 2

    def test_delete_activities_by_filter(self):
        # Create activities
        self.service.log_activity(Context.from_trace_id('123'), self.activity1)
        self.service.log_activity(Context.from_trace_id('123'), self.activity2)
        self.service.log_activity(Context.from_trace_id('123'), self.activity3)

        # Delete activities by party id
        self.service.delete_activities_by_filter(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('party_id', '1')
        )

        # Get remaining activities
        page = self.service.get_activities(
            Context.from_trace_id('123'),
            FilterParams(),
            PagingParams()
        )
        assert len(page.data) == 1
        assert page.data[0].id == self.activity3.id 


# -*- coding: utf-8 -*-
from typing import Optional


class ReferenceV1:
    """
    Reference object used to identify a related object.
    
    Attributes:
        id (str): Unique identifier of the referenced object
        type (str): Type of the referenced object
        name (str): Human-readable name of the referenced object
    """
    
    def __init__(self, id: str = None, type: str = None, name: Optional[str] = None):
        """
        Creates a new reference to an object
        
        Args:
            id: Unique identifier of the referenced object
            type: Type of the referenced object
            name: Human-readable name of the referenced object
        """
        self.id = id
        self.type = type
        self.name = name


# -*- coding: utf-8 -*-
import json
import bottle

from typing import Optional, Any
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context
from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema

from eic_aichat_users.sessions.data import SessionV1
from eic_aichat_users.sessions.data.SessionV1Schema import SessionV1Schema
from eic_aichat_users.sessions.logic.ISessionsService import ISessionsService


class SessionsOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._service: ISessionsService = None
        self._dependency_resolver.put("sessions-service", Descriptor("aichatusers-sessions", "service", "*", "*", "1.0"))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._service = self._dependency_resolver.get_one_required("sessions-service")

    def get_sessions(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._service.get_sessions(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_session_by_id(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._service.get_session_by_id(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def open_session(self):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            res = self._service.open_session(
                context,
                data.get("user_id"),
                data.get("user_name"),
                data.get("address"),
                data.get("client"),
                data.get("user"),
                data.get("data"),
            )
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def store_session_data(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            res = self._service.store_session_data(context, session_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def update_session_user(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        data = bottle.request.json or {}
        try:
            res = self._service.update_session_user(context, session_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def close_session(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._service.close_session(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def delete_session_by_id(self, session_id):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            res = self._service.delete_session_by_id(context, session_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def close_expired_sessions(self):
        context = Context.from_trace_id(self._get_trace_id())
        try:
            self._service.close_expired_sessions(context)
            return self._send_empty_result()
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController):
        controller.register_route("get", "/sessions", None, self.get_sessions)

        controller.register_route("get", "/sessions/<session_id>", ObjectSchema(True)
                                  .with_required_property("session_id", TypeCode.String),
                                  self.get_session_by_id)

        controller.register_route("post", "/sessions/open", ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String)
                                  .with_required_property("user_name", TypeCode.String),
                                  self.open_session)

        controller.register_route("post", "/sessions/<session_id>/data", ObjectSchema(True)
                                  .with_required_property("session_id", TypeCode.String),
                                  self.store_session_data)

        controller.register_route("post", "/sessions/<session_id>/user", ObjectSchema(True)
                                  .with_required_property("session_id", TypeCode.String),
                                  self.update_session_user)

        controller.register_route("post", "/sessions/<session_id>/close", ObjectSchema(True)
                                  .with_required_property("session_id", TypeCode.String),
                                  self.close_session)

        controller.register_route("delete", "/sessions/<session_id>", ObjectSchema(True)
                                  .with_required_property("session_id", TypeCode.String),
                                  self.delete_session_by_id)

        controller.register_route("post", "/sessions/cleanup", None, self.close_expired_sessions)



# -*- coding: utf-8 -*-
from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema, MapSchema
from pip_services4_data.validate import ArraySchema

from .ReferenceV1Schema import ReferenceV1Schema


class PartyActivityV1Schema(ObjectSchema):
    """
    Schema for validating PartyActivityV1 objects
    """
    
    def __init__(self):
        super().__init__()
        
        self.with_optional_property('id', TypeCode.String)
        self.with_optional_property('time', TypeCode.DateTime)
        self.with_required_property('type', TypeCode.String)
        self.with_required_property('party', ReferenceV1Schema())
        self.with_optional_property('ref_item', ReferenceV1Schema())
        self.with_optional_property('ref_parents', ArraySchema(ReferenceV1Schema()))
        self.with_optional_property('ref_party', ReferenceV1Schema())
        self.with_optional_property('details', MapSchema(TypeCode.String, TypeCode.String)) 


# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext

from ..data.version1.UserPasswordInfoV1 import UserPasswordInfoV1


class IPasswordsService(ABC):

    @abstractmethod
    def get_password_info(self, context: Optional[IContext], user_id: str) -> UserPasswordInfoV1:
        raise NotImplementedError()

    @abstractmethod
    def validate_password(self, context: Optional[IContext], password: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def validate_password_for_user(self, context: Optional[IContext], user_id: str, password: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def set_password(self, context: Optional[IContext], user_id: str, password: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def set_temp_password(self, context: Optional[IContext], user_id: str) -> str:
        raise NotImplementedError()

    @abstractmethod
    def delete_password(self, context: Optional[IContext], user_id: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def authenticate(self, context: Optional[IContext], user_id: str, password: str) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def change_password(self, context: Optional[IContext], user_id: str, old_password: str, new_password: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def validate_code(self, context: Optional[IContext], user_id: str, code: str) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def reset_password(self, context: Optional[IContext], user_id: str, code: str, password: str) -> None:
        raise NotImplementedError()

    @abstractmethod
    def recover_password(self, context: Optional[IContext], user_id: str) -> None:
        raise NotImplementedError()
